#include "program.h"

void _entryPoint()
{
	/****************************>           Get Handles           <****************************/
	//Get a handle to coreinit.rpl
	unsigned int coreinit_handle;
	OSDynLoad_Acquire("coreinit.rpl", &coreinit_handle);
	//Get a handle to vpad.rpl */
	unsigned int vpad_handle;
	OSDynLoad_Acquire("vpad.rpl", &vpad_handle);
	/****************************>       External Prototypes       <****************************/
	//VPAD functions
	int(*VPADRead)(int controller, VPADData *buffer, unsigned int num, int *error);
	//OS functions
	void(*_Exit)();
	/****************************>             Exports             <****************************/
	//VPAD functions
	OSDynLoad_FindExport(vpad_handle, 0, "VPADRead", &VPADRead);
	//OS functions
	OSDynLoad_FindExport(coreinit_handle, 0, "_Exit", &_Exit);
	/****************************>             Globals             <****************************/
	struct renderFlags flags;
	flags.start=1;
	flags.pow=1;
	flags.touch=1;
	flags.y=0;
	flags.x=0;
	flags.a=0;
	flags.b=0;
	flags.l=0;
	flags.r=0;
	flags.zl=0;
	flags.zr=0;
	flags.du=0;
	flags.dd=0;
	flags.dl=0;
	flags.dr=0;
	flags.plus=0;
	flags.minus=0;
	
	VPADData vpad_data;
	
	__os_snprintf(flags.aPressed, 32, "A button pressed");
	__os_snprintf(flags.bPressed, 32, "B button pressed");
	__os_snprintf(flags.xPressed, 32, "X button pressed");
	__os_snprintf(flags.yPressed, 32, "Y button pressed");
	__os_snprintf(flags.lPressed, 32, "L button pressed");
	__os_snprintf(flags.rPressed, 32, "R button pressed");
	__os_snprintf(flags.zlPressed, 32, "ZL button pressed");
	__os_snprintf(flags.zrPressed, 32, "ZR button pressed");
	__os_snprintf(flags.dUp, 32, "D-PAD UP button pressed");
	__os_snprintf(flags.dDown, 32, "D-PAD DOWN button pressed");
	__os_snprintf(flags.dLeft, 32, "D-PAD LEFT button pressed");
	__os_snprintf(flags.dRight, 32, "D-PAD RIGHT button pressed");
	__os_snprintf(flags.plusPressed, 32, "+ (plus) button pressed");
	__os_snprintf(flags.minusPressed, 32, "- (minus) button pressed");
	/****************************>            VPAD Loop            <****************************/

	/* Enter the VPAD loop */
	int error;
	//Read initial vpad status
	VPADRead(0, &vpad_data, 1, &error);

	while(1)
	{
		VPADRead(0, &vpad_data, 1, &error);
		
		float lStickStorageX = vpad_data.lstick.x;
		float lStickStorageY = vpad_data.lstick.y;
		float rStickStorageX = vpad_data.rstick.x;
		float rStickStorageY = vpad_data.rstick.y;
		
		uint8_t batteryVal = vpad_data.battery;
		
		__os_snprintf(flags.startUp, 255, "Mii-Test-U - A Wii U functions tester by Voxel Studios.\n\nLStick:\n%f\n%f\nRStick:\n%f\n%f\n\nButton Tester:\n\n\nBattery:\n\n\nTouch Coords:\n\n\n\n\n\n\n\n\n\n\nv0.1", lStickStorageX, lStickStorageY, rStickStorageX, rStickStorageY);
		
		if(batteryVal==0) {
			__os_snprintf(flags.power, 32, "Charging...");
		} else if(batteryVal==1) {
			__os_snprintf(flags.power, 32, "|");
		} else if(batteryVal==2) {
			__os_snprintf(flags.power, 32, "||");
		} else if(batteryVal==3) {
			__os_snprintf(flags.power, 32, "|||");
		} else if(batteryVal==4) {
			__os_snprintf(flags.power, 32, "||||");
		} else if(batteryVal==5) {
			__os_snprintf(flags.power, 32, "|||||");
		} else if(batteryVal==6) {
			__os_snprintf(flags.power, 32, "||||||");
		}
		
		if (vpad_data.tpdata.touched == 1)
		{
			__os_snprintf(flags.touching, 32, "%i,%i", vpad_data.tpdata.x, vpad_data.tpdata.y);
		}

		//button A
		if (vpad_data.btn_hold & BUTTON_A)
			flags.a=1;
	
		//button B
		if (vpad_data.btn_hold & BUTTON_B)
			flags.b=1;

		//button X
		if (vpad_data.btn_hold & BUTTON_X)
			flags.x=1;

		//button Y
		if (vpad_data.btn_hold & BUTTON_Y)
			flags.y=1;

		if (vpad_data.btn_hold & BUTTON_L)
			flags.l=1;
			
		if (vpad_data.btn_hold & BUTTON_R)
			flags.r=1;
			
		if (vpad_data.btn_hold & BUTTON_ZL)
			flags.zl=1;
			
		if (vpad_data.btn_hold & BUTTON_ZR)
			flags.zr=1;
		
		if (vpad_data.btn_hold & BUTTON_UP)
			flags.du=1;
			
		if (vpad_data.btn_hold & BUTTON_DOWN)
			flags.dd=1;
		
		if (vpad_data.btn_hold & BUTTON_LEFT)
			flags.dl=1;
			
		if (vpad_data.btn_hold & BUTTON_RIGHT)
			flags.dr=1;
			
		if (vpad_data.btn_hold & BUTTON_PLUS)
			flags.plus=1;
			
		if (vpad_data.btn_hold & BUTTON_MINUS)
			flags.minus=1;

		//end
		if(vpad_data.btn_hold & BUTTON_HOME)
			break;

		render(&flags);
	}
	
	
	//WARNING: DO NOT CHANGE THIS. YOU MUST CLEAR THE FRAMEBUFFERS AND IMMEDIATELY CALL EXIT FROM THIS FUNCTION. RETURNING TO LOADER CAUSES FREEZE.
	int ii=0;
	for(ii;ii<2;ii++)
	{
		fillScreen(0,0,0,0);
		flipBuffers();
	}
	_Exit();
}

void render(struct renderFlags *flags)
{
	int i=0;
	for(i;i<2;i++)
	{
		fillScreen(0,0,0,0);
		if(flags->start)
			drawString(0,0,flags->startUp);
		if(flags->a)
			drawString(0,10,flags->aPressed);
		if(flags->b)
			drawString(0,10,flags->bPressed);
		if(flags->x)
			drawString(0,10,flags->xPressed);
		if(flags->y)
			drawString(0,10,flags->yPressed);
		if(flags->l)
			drawString(0,10,flags->lPressed);
		if(flags->r)
			drawString(0,10,flags->rPressed);
		if(flags->zl)
			drawString(0,10,flags->zlPressed);
		if(flags->zr)
			drawString(0,10,flags->zrPressed);
		if(flags->plus)
			drawString(0,10,flags->plusPressed);
		if(flags->minus)
			drawString(0,10,flags->minusPressed);
		if(flags->pow)
			drawString(0,13,flags->power);
		if(flags->touch)
			drawString(0,16,flags->touching);
		if(flags->du)
			drawString(0,10,flags->dUp);
		if(flags->dd)
			drawString(0,10,flags->dDown);
		if(flags->dl)
			drawString(0,10,flags->dLeft);
		if(flags->dr)
			drawString(0,10,flags->dRight);
		flipBuffers();
	}
	flags->start=1;
	flags->pow=1;
	flags->touch=1;
	flags->a=0;
	flags->b=0;
	flags->x=0;
	flags->y=0;
	flags->l=0;
	flags->r=0;
	flags->zl=0;
	flags->zr=0;
	flags->plus=0;
	flags->minus=0;
	flags->du=0;
	flags->dd=0;
	flags->dl=0;
	flags->dr=0;
}